/**
 * SqlDb
 * Refer to: https://github.com/psayre23/WebSQL
 * User: Amiku Zhang <amzhang@actiontec.com>
 * Date: 2015-03-03 14:50
 */

define(function (require, exports, module) {
    require('./src/jquery.websql.min');
    var db = null;

    function SqlDb() {
        "use strict";
        //alert("contructor...");
        db = $.WebSQL('todonotedb');
        this.initDb();
    }

    SqlDb.prototype.initDb = function () {
        db.query(
            // Create table todolist
            'CREATE TABLE IF NOT EXISTS todolist (id INTEGER PRIMARY KEY AUTOINCREMENT, title, content, createdtime, updatedtime,datetimeunix, datetimereadable, date, time, priority, isdone, isarchived, istrash, parentid, password)',
            // createdtime

            // Create table settings
            'CREATE TABLE IF NOT EXISTS settings (id INTEGER PRIMARY KEY AUTOINCREMENT, key, val, comment)',

            // Create index
            'CREATE INDEX IF NOT EXISTS todolist__done ON todolist (content)',

            // Select todolist not done
            function () {
                // 如果有需要增加的字段或者数据库的回调，在这里编写 | 阿米酷<i@amiku.cn> 2016-04-03 11:51
                // 最终：统一移到外部todo.js里面处理
                // 'ALTER TABLE todolist ADD COLUMN password'
                // Version 1.1.8 SQL：增加的字段
                // 'ALTER TABLE todolist ADD COLUMN createdtime;UPDATE todolist set createdtime=updatedtime WHERE createdtime==\'1\';'
                // "UPDATE todolist set createdtime=updatedtime FROM todolist WHERE createdtime='1'"
                // "UPDATE todolist set createdtime=updatedtime FROM todolist WHERE createdtime='' or createdtime='1' or  createdtime= null"
                // "UPDATE todolist set createdtime5=updatedtime WHERE createdtime5==''"
                // "UPDATE todolist set createdtime=todolist.updatedtime FROM todolist WHERE createdtime='' or  createdtime= null",

                /*!
                 SELECT id, content
                 FROM todolist
                 WHERE isdone IS NULL
                 ORDER BY id DESC
                 */
            }
        ).fail(function (tx, err) {
            console.log('An error occurred while loading the todolist');
            throw new Error(err.message);
        }).done(function (todos) {
            //// Create todos html
            //var items = [];
            //for(var i = 0; i < todos.length; i++) {
            //    items[i] = todoItemHTML(todos[i].id, todos[i].content);
            //}
            //
            //// Create list from html
            //$('#todos').html(items.join(''));
        });
    };

    //SqlDb.prototype.select = function (table, fn) {
    SqlDb.prototype.query = function (sql, fn) {
        //db.query('SELECT "Wow, jQuery, cool!" AS msg')
        //    .done(function (rows) {
        //        alert(rows[0].msg);
        //    });

        //var sql = 'SELECT * FROM '+table;
        //alert(sql);
        //db.query('SELECT * FROM products WHERE category_id = ?', [catID])
        db.query(sql).fail(function (ex, err) {
            console.log('An error occurred while querying your table:' + sql);
            throw new Error(err.message);
        }).done(function (res) {
            if (fn && typeof(fn) == "function") {
                fn(res);
            }
            //fn(res);
        });
    };

    SqlDb.prototype.insert = function (table, fields, value_models, values, fn) {
        var sql = 'INSERT INTO ' + table + ' (' + fields + ') VALUES (' + value_models + ')';
        //alert(sql);
        db.query(sql, values)
            .fail(function (tx, err) {
                console.log('An error occurred while inserting your ' + table + ':' + sql);
                throw new Error(err.message);
            })
            .done(function (res) {
                if (fn && typeof(fn) == "function") {
                    fn(res);
                }
                //fn(res);
            });
    };

    SqlDb.prototype.update = function () {

    };
    /**
     * 统一从数据库中读取处理
     * @param str
     * @returns {*}
     */
    SqlDb.prototype.getFromDb = function (str) {
        //console.log("Amiku Test getFromDb");
        //console.log(str);

        //var str = "A'B'A'C'B";
        //str = str.replace(/\'/g, "'");
        //str = str.replace(/\\'/g,"dd");
        //str = str.replace(/'/g, "");
        //console.log(str);
        //console.log(str.replace(/\\'/g, "'"));
        return str;
    };

    /**
     * 统一设置到数据库中
     * @param str
     * @returns {*}
     */
    SqlDb.prototype.setToDb = function (str) {
        //console.log(str);
        //str = str.replace(/'/g, "\'");
        //str = str.replace(/'/g, "/u0027");
        //str = str.replace(/\'/g,"\\'");
        str = str.replace(/\'/g, "&#39;");

        //str = str.replace(/'/g, "\\'");
        //console.log(str.replace(/\\'/g, "'"));
        return str;
    };
    SqlDb.prototype.empty = function () {
        //alert("empty TABLE AND INDEX...");
        db.query(
            'DROP TABLE IF EXISTS todolist',
            'DROP TABLE IF EXISTS settings',
            'DROP INDEX IF EXISTS todolist__done'
        ).fail(function (tx, err) {
            console.log('An error occurred while empty the table and index');
            throw new Error(err.message);
        }).done(function () {
        });
    };

    module.exports = SqlDb;
});
